/*
 * tmos - port.h 
 * Copyright (C) 2009  Krzysztof Mazur <krzysiek@podlesie.net>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software Fundation,
 * Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
 */

#ifndef __TMOS_PORT_H
#define __TMOS_PORT_H

#include <tmos/kernel.h>
#include <tmos/types.h>
#include <tmos/jiffies.h>

#ifdef CONFIG_PORT
typedef signed char msg_ssize_t;
typedef unsigned char msg_size_t;
typedef unsigned char msg_flags_t;

#define O_RDONLY		0
#define O_WRONLY		0
#define O_RDWR			0

#define POLLIN		0x01
#define POLLOUT		0x02

#if CONFIG_OFFSET_32
typedef int32_t off_t;
#define OFF_MAX INT32_MAX
#elif CONFIG_OFFSET_16
typedef int16_t off_t;
#define OFF_MAX INT16_MAX
#elif CONFIG_OFFSET_8
typedef int8_t off_t;
#define OFF_MAX INT8_MAX
#else
#error "unsupported offset size"
#endif

#define MSG_DONTWAIT		1

typedef signed char port_t;

struct iocb {
	off_t off;
	port_t port;
	char __kernel *buf;
	msg_size_t len;
};

typedef msg_ssize_t (*port_read_t)(struct iocb __kernel *iocb) __reentrant;
extern const port_read_t port_read_table[];

typedef msg_ssize_t (*port_write_t)(struct iocb __kernel *iocb) __reentrant;
extern const port_write_t port_write_table[];

typedef unsigned char (*port_poll_t)(port_t port)
	__reentrant;
extern const port_poll_t port_poll_table[];

port_t __port_open(const char *__kernel name, int flags) __reentrant;
#define port_open(name, flags, ...) __port_open(name, flags)

port_t __port_open_c(const char *__kcode name, int flags) __reentrant;
#define port_open_c(name, flags, ...) __port_open(name, flags)

msg_ssize_t port_readi(struct iocb __kernel *iocb) __reentrant;
msg_ssize_t port_writei(struct iocb __kernel *iocb) __reentrant;

msg_ssize_t port_pread(port_t port, char __kernel *buf, msg_size_t len,
		off_t off)
	__reentrant;

msg_ssize_t port_pwrite(port_t port, char __kernel *buf, msg_size_t len,
		off_t off)
	__reentrant;

#define __port_poll(port) \
	port_poll_table[port](port)

#define port_read(port, buf, len) \
	port_pread(port, buf, len, 0)

#define port_write(port, buf, len) \
	port_pwrite(port, buf, len, 0)

msg_ssize_t port_read_timeout(port_t port, char __kernel *buf, msg_size_t len,
		jiffies_t timeout) __reentrant;

#define PORT_2(path, read, write, poll) \
	msg_ssize_t read(struct iocb __kernel *) \
			__reentrant;
#include <tmos/ports.h>
#undef PORT_2

#define PORT_2(path, read, write, poll) \
	msg_ssize_t write(struct iocb __kernel *) \
			__reentrant;
#include <tmos/ports.h>
#undef PORT_2

#define PORT_2(path, read, write, poll) \
	unsigned char poll(port_t) __reentrant;
#include <tmos/ports.h>
#undef PORT_2

#endif

#endif
