/*
 * tmos - port.c
 * Copyright (C) 2009  Krzysztof Mazur <krzysiek@podlesie.net>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software Fundation,
 * Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
 */

#include <tmos/kernel.h>
#include <tmos/port.h>
#include <tmos/string.h>
#include <tmos/sched.h>
#include <tmos/errno.h>

static msg_ssize_t dev_write_inval(struct iocb __kernel *iocb) __reentrant
{
	return -EINVAL;
}

unsigned char dev_poll_generic(port_t port) __reentrant
{
	return POLLIN | POLLOUT;
}

#if CONFIG_DEV_NULL
static msg_ssize_t dev_null_read(struct iocb __kernel *iocb) __reentrant
{
	return 0;
}

static msg_ssize_t dev_null_write(struct iocb __kernel *iocb) __reentrant
{
	return iocb->len;
}
#endif

#if CONFIG_DEV_ZERO
static msg_ssize_t dev_zero_read(struct iocb __kernel *iocb) __reentrant
{
	msg_size_t len = iocb->len;

	memset_kernel(iocb->buf, 0, len);
	return len;
}
#endif

#define PORT_2(path, read, write, poll)		(path),
static const char *__code port_path_table[] = {
#include <tmos/ports.h>
};
#undef PORT_2

#define PORT_2(path, read, write, poll)		(read),
const port_read_t port_read_table[] = {
#include <tmos/ports.h>
};
#undef PORT_2

#define PORT_2(path, read, write, poll)		(write),
const port_write_t port_write_table[] = {
#include <tmos/ports.h>
};
#undef PORT_2

#define PORT_2(path, read, write, poll)		(poll),
const port_poll_t port_poll_table[] = {
#include <tmos/ports.h>
};
#undef PORT_2

/*
 * TODO: add filesystem support
 */
port_t __port_open(const char *name, int flags) __reentrant
{
	unsigned char i;

	for (i = 0; i < ARRAY_SIZE(port_path_table); i++)
		if (!strcmp(name, port_path_table[i]))
			return i;

	return (port_t) -1;
}

msg_ssize_t port_readi(struct iocb __kernel *iocb) __reentrant
{
	return port_read_table[iocb->port](iocb);
}

msg_ssize_t port_writei(struct iocb __kernel *iocb) __reentrant
{
	return port_write_table[iocb->port](iocb);
}

msg_ssize_t port_pread(port_t port, char __kernel *buf, msg_size_t len,
		off_t off)
	__reentrant
{
	struct iocb iocb;

	iocb.port = port;
	iocb.buf = buf;
	iocb.len = len;
	iocb.off = off;
	return port_readi(&iocb);
}

msg_ssize_t port_pwrite(port_t port, char __kernel *buf, msg_size_t len,
		off_t off)
	__reentrant
{
	struct iocb iocb;

	iocb.port = port;
	iocb.buf = buf;
	iocb.len = len;
	iocb.off = off;
	return port_writei(&iocb);
}
